<?php
/**
 * Post Date Time Change
 * 
 * @package    Post Date Time Change
 * @subpackage Post Date Time Change Main & Management screen
/*  Copyright (c) 2014- Katsushi Kawamori (email : dodesyoswift312@gmail.com)
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 2 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

class PostDateTimeChangeAdmin {

	/* ==================================================
	 * Add a "Settings" link to the plugins page
	 * @since	1.0
	 */
	function settings_link( $links, $file ) {
		static $this_plugin;
		if ( empty($this_plugin) ) {
			$this_plugin = POSTDATETIMECHANGE_PLUGIN_BASE_FILE;
		}
		if ( $file == $this_plugin ) {
			$links[] = '<a href="'.admin_url('options-general.php?page=postdatetimechange').'">'.__( 'Settings').'</a>';
		}
			return $links;
	}

	/* ==================================================
	 * Settings page
	 * @since	1.0
	 */
	function add_pages() {
		add_options_page( 'Post Date Time Change Options', 'Post Date Time Change', 'edit_posts', 'postdatetimechange', array($this, 'manage_page') );
	}

	/* =================================================
	 * Add Css and Script
	 * @since	1.0
	 */
	function load_custom_wp_admin_style() {
		if ($this->is_my_plugin_screen()) {
			if(get_option('postdatetimechange_picker')) {
				wp_enqueue_style( 'jquery-datetimepicker', POSTDATETIMECHANGE_PLUGIN_URL.'/css/jquery.datetimepicker.css' );
			}
			wp_enqueue_script( 'jquery' );
			if(get_option('postdatetimechange_picker')) {
				wp_enqueue_script( 'jquery-datetimepicker', POSTDATETIMECHANGE_PLUGIN_URL.'/js/jquery.datetimepicker.js', null, '2.3.4' );
				wp_enqueue_script( 'jquery-postdatetimechange-datetimepicker', POSTDATETIMECHANGE_PLUGIN_URL.'/js/jquery.postdatetimechange.datetimepicker.js', array('jquery') );
			}
			wp_enqueue_script( 'postdatetimechange-js', POSTDATETIMECHANGE_PLUGIN_URL.'/js/jquery.postdatetimechange.js', array('jquery') );
		}
	}

	/* ==================================================
	 * For only admin style
	 * @since	1.0
	 */
	function is_my_plugin_screen() {
		$screen = get_current_screen();
		if ( isset($_REQUEST['post_type']) ) {
			$posttype = sanitize_text_field($_REQUEST['post_type']);
		} else {
			$posttype = 'post';
		}
		if (is_object($screen) && $screen->id == 'tools_page_postdatetimechange') {
			return TRUE;
		} else if (is_object($screen) && $screen->id == 'edit-'.$posttype) {
			return TRUE;
		} else if (is_object($screen) && $screen->id == 'upload') {
			return TRUE;
		} else {
			return FALSE;
		}
	}

	/* ==================================================
	 * Main
	 */
	function manage_page() {

		if ( !current_user_can( 'edit_posts' ) )  {
			wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
		}

		$plugin_datas = get_file_data( POSTDATETIMECHANGE_PLUGIN_BASE_DIR.'/postdatetimechange.php', array('version' => 'Version') );
		$plugin_version = __('Version:').' '.$plugin_datas['version'];
		$method = get_option('postdatetimechange_method');
		$write = get_option('postdatetimechange_write');

		?>
		<div class="wrap">
		<h2>Post Date Time Change</h2>

			<form method="post" action="options.php">
				<?php settings_fields('postdatetimechange_settings'); ?>
				<div style="width: 250px; margin: 5px; padding: 5px; border: #CCC 2px solid;">
				<h3 style="margin: 5px; padding: 5px;"><?php _e('View'); ?></h3>
				<div style="display: block;padding:5px 5px"><input type="checkbox" name="postdatetimechange_picker" value="1" <?php checked('1', get_option('postdatetimechange_picker')); ?> />DateTimePicker</div>
				<div style="display: block;padding:5px 5px"><input type="radio" id="postdatetimechange_method" name="postdatetimechange_method" value="posted" <?php if ($method === 'posted') echo 'checked'; ?>><?php _e('Posted', 'post-date-time-change'); ?></div>
				<div style="display: block;padding:5px 5px"><input type="radio" id="postdatetimechange_method" name="postdatetimechange_method" value="modified" <?php if ($method === 'modified') echo 'checked'; ?>><?php _e('Last updated'); ?></div>
				<h3 style="margin: 5px; padding: 5px;"><?php _e('Change'); ?></h3>
				<div style="display: block;padding:5px 5px"><input type="radio" id="postdatetimechange_write" name="postdatetimechange_write" value="date_modified" <?php if ($write === 'date_modified') echo 'checked'; ?>><?php _e('Posted And Modified', 'post-date-time-change'); ?></div>
				<div style="display: block;padding:5px 5px"><input type="radio" id="postdatetimechange_write" name="postdatetimechange_write" value="date" <?php if ($write === 'date') echo 'checked'; ?>><?php _e('Only Posted', 'post-date-time-change'); ?></div>
				<div style="display: block;padding:5px 5px"><input type="radio" id="postdatetimechange_write" name="postdatetimechange_write" value="modified" <?php if ($write === 'modified') echo 'checked'; ?>><?php _e('Only Modified', 'post-date-time-change'); ?></div>
				<?php submit_button( __('Save Changes'), 'primary', 'Submit', TRUE ); ?>
				</div>
				<div style="clear:both"></div>
			</form>

			<?php $this->credit(); ?>

		</div>
		<?php

	}

	/* ==================================================
	 * Credit
	 */
	function credit() {

		$plugin_name = NULL;
		$plugin_ver_num = NULL;
		$plugin_path = plugin_dir_path( __DIR__ );
		$plugin_dir = untrailingslashit($plugin_path);
		$slugs = explode('/', $plugin_dir);
		$slug = end($slugs);
		$files = scandir($plugin_dir);
		foreach ($files as $file) {
			if($file == '.' || $file == '..' || is_dir($plugin_path.$file)){
				continue;
			} else {
				$exts = explode('.', $file);
				$ext = strtolower(end($exts));
				if ( $ext === 'php' ) {
					$plugin_datas = get_file_data( $plugin_path.$file, array('name'=>'Plugin Name', 'version' => 'Version') );
					if ( array_key_exists( "name", $plugin_datas ) && !empty($plugin_datas['name']) && array_key_exists( "version", $plugin_datas ) && !empty($plugin_datas['version']) ) {
						$plugin_name = $plugin_datas['name'];
						$plugin_ver_num = $plugin_datas['version'];
						break;
					}
				}
			}
		}
		$plugin_version = __('Version:').' '.$plugin_ver_num;
		$faq = __('https://wordpress.org/plugins/'.$slug.'/faq', $slug);
		$support = 'https://wordpress.org/support/plugin/'.$slug;
		$review = 'https://wordpress.org/support/view/plugin-reviews/'.$slug;
		$translate = 'https://translate.wordpress.org/projects/wp-plugins/'.$slug;
		$facebook = 'https://www.facebook.com/katsushikawamori/';
		$twitter = 'https://twitter.com/dodesyo312';
		$youtube = 'https://www.youtube.com/channel/UC5zTLeyROkvZm86OgNRcb_w';
		$donate = __('https://riverforest-wp.info/donate/', $slug);

		?>
		<span style="font-weight: bold;">
		<div>
		<?php echo $plugin_version; ?> | 
		<a style="text-decoration: none;" href="<?php echo $faq; ?>" target="_blank"><?php _e('FAQ'); ?></a> | <a style="text-decoration: none;" href="<?php echo $support; ?>" target="_blank"><?php _e('Support Forums'); ?></a> | <a style="text-decoration: none;" href="<?php echo $review; ?>" target="_blank"><?php _e('Reviews', 'media-from-ftp'); ?></a>
		</div>
		<div>
		<a style="text-decoration: none;" href="<?php echo $translate; ?>" target="_blank"><?php echo sprintf(__('Translations for %s'), $plugin_name); ?></a> | <a style="text-decoration: none;" href="<?php echo $facebook; ?>" target="_blank"><span class="dashicons dashicons-facebook"></span></a> | <a style="text-decoration: none;" href="<?php echo $twitter; ?>" target="_blank"><span class="dashicons dashicons-twitter"></span></a> | <a style="text-decoration: none;" href="<?php echo $youtube; ?>" target="_blank"><span class="dashicons dashicons-video-alt3"></span></a>
		</div>
		</span>

		<div style="width: 250px; height: 180px; margin: 5px; padding: 5px; border: #CCC 2px solid;">
		<h3><?php _e('Please make a donation if you like my work or would like to further the development of this plugin.', $slug); ?></h3>
		<div style="text-align: right; margin: 5px; padding: 5px;"><span style="padding: 3px; color: #ffffff; background-color: #008000">Plugin Author</span> <span style="font-weight: bold;">Katsushi Kawamori</span></div>
		<button type="button" style="margin: 5px; padding: 5px;" onclick="window.open('<?php echo $donate; ?>')"><?php _e('Donate to this plugin &#187;'); ?></button>
		</div>

		<?php

	}

	/* ==================================================
	 * Post Column
	 * @param	array	$cols
	 * @return	array	$cols
	 * @since	4.00
	 */
	function ptuc_column( $cols ) {
		$method = get_option('postdatetimechange_method');
		if ($method === 'modified') {
			$cols["post_date_and_time"] = __('Date and time').' '.__('Last updated');
		} else {
			$cols["post_date_and_time"] = __('Date and time').' '.__('Posted', 'post-date-time-change');
		}
		return $cols;
	}

	/* ==================================================
	 * Post Column
	 * @param	string	$column_name
	 * @param	int		$id
	 * @since	4.00
	 */
	function ptuc_value( $column_name, $id ) {
		if ( $column_name == "post_date_and_time" ) {
			echo $this->input_date_time($id);
		}
	}

	/* ==================================================
	 * Page Column
	 * @param	array	$cols
	 * @return	array	$cols
	 * @since	4.00
	 */
	function pguc_column( $cols ) {
		$method = get_option('postdatetimechange_method');
		if ($method === 'modified') {
			$cols["post_date_and_time"] = __('Date and time').' '.__('Last updated');
		} else {
			$cols["post_date_and_time"] = __('Date and time').' '.__('Posted', 'post-date-time-change');
		}
		return $cols;
	}

	/* ==================================================
	 * Page Column
	 * @param	string	$column_name
	 * @param	int		$id
	 * @since	4.00
	 */
	function pguc_value( $column_name, $id ) {
		if ( $column_name == "post_date_and_time" ) {
			echo $this->input_date_time($id);
		}
	}

	/* ==================================================
	 * Media Library Column
	 * @param	array	$cols
	 * @return	array	$cols
	 * @since	4.00
	 */
	function muc_column( $cols ) {
		$method = get_option('postdatetimechange_method');
		if ($method === 'modified') {
			$cols["media_date_and_time"] = __('Date and time').' '.__('Last updated');
		} else {
			$cols["media_date_and_time"] = __('Date and time').' '.__('Posted', 'post-date-time-change');
		}
		return $cols;
	}

	/* ==================================================
	 * Media Library Column
	 * @param	string	$column_name
	 * @param	int		$id
	 * @since	4.00
	 */
	function muc_value( $column_name, $id ) {
		if ( $column_name == "media_date_and_time" ) {
			echo $this->input_date_time($id);
		}
	}

	/* ==================================================
	 * Input Date Time
	 * @param	int		$id
	 * @return	string	$input_html
	 * @since	4.00
	 */
	function input_date_time($id) {

		$method = get_option('postdatetimechange_method');

		global $wpdb;
		$attachments = $wpdb->get_results($wpdb->prepare("
					SELECT	post_date, post_modified
					FROM	$wpdb->posts
					WHERE	ID = %d
					",$id
					),ARRAY_A);
		if ($method === 'modified') {
			$date = $attachments[0]['post_modified'];
		} else {
			$date = $attachments[0]['post_date'];
		}
		$newdate = substr( $date , 0 , strlen($date)-3 );

		$input_html = '<input type="text" id="datetimepicker-postdatetimechange'.$id.'" name="postdatetimechange_datetime['.$id.']" value="'.$newdate.'" style="width: 100%;" />';

		return $input_html;

	}

	/* ==================================================
	 * Bulk Action Select
	 * @since	4.00
	 */
	function custom_bulk_admin_footer() {
		global $pagenow;
		if($pagenow == 'upload.php' || $pagenow == 'edit.php') {

			$now_date_time = date_i18n("Y-m-d H:i");
			$html = '<input type="text" id="datetimepicker-postdatetimechange0" name="bulk_postdatetimechange_datetime" value="'.$now_date_time.'" style="width: 100%;" />';

			?>
			<script type="text/javascript">
				jQuery(document).ready(function() {
					jQuery('<option>').val('changedatetime').text('<?php _e('Change the date and time', 'post-date-time-change')?>').appendTo("select[name='action']");
					jQuery('<option>').val('changedatetime').text('<?php _e('Change the date and time', 'post-date-time-change')?>').appendTo("select[name='action2']");
				});
				jQuery('<?php echo $html; ?>').appendTo("#post_date_and_time");
				jQuery('<?php echo $html; ?>').appendTo("#media_date_and_time");
			</script>
			<?php
		}
	}

	/* ==================================================
	 * Bulk Action for post & page
	 * @since	4.00
	 */
	function custom_bulk_action() {

		// get the action
		$wp_list_table = _get_list_table('WP_Posts_List_Table');  
		$action = $wp_list_table->current_action();

		$allowed_actions = array("changedatetime");
		if(!in_array($action, $allowed_actions)) return;

		check_admin_referer('bulk-posts');

		if(isset($_REQUEST['post'])) {
			$post_ids = array_map('intval', $_REQUEST['post']);
		}

		if(empty($post_ids)) return;

		$sendback = remove_query_arg( array('datetimechanged', 'message', 'untrashed', 'deleted', 'ids'), wp_get_referer() );
		if ( ! $sendback )
		$sendback = admin_url( "edit.php?post_type=$post_type" );

		$pagenum = $wp_list_table->get_pagenum();
		$sendback = add_query_arg( 'paged', $pagenum, $sendback );

		switch($action) {
			case 'changedatetime':
				$write = get_option('postdatetimechange_write');
				$datetimechanged = 0;
				$postdatetimechange_datetimes = $this->sanitize_array($_REQUEST['postdatetimechange_datetime']);

				global $wpdb;
				$messages = array();
				foreach( $post_ids as $post_id ) {
					$postdate = $postdatetimechange_datetimes[$post_id].':00';
					$postdategmt = get_gmt_from_date($postdate);
					// Change DB Attachement post
					$message = 'success';
					if ($write === 'date') {
						$update_array = array(
										'post_date' => $postdate,
										'post_date_gmt' => $postdategmt
									);
					}else if ($write === 'modified') {
						$update_array = array(
										'post_modified' => $postdate,
										'post_modified_gmt' => $postdategmt
									);
					} else {
						$update_array = array(
										'post_date' => $postdate,
										'post_date_gmt' => $postdategmt,
										'post_modified' => $postdate,
										'post_modified_gmt' => $postdategmt
									);
					}
					$id_array= array('ID'=> $post_id);
					$wpdb->show_errors();
					$wpdb->update( $wpdb->posts, $update_array, $id_array, array('%s'), array('%d') );

					if ( $wpdb->last_error !== '' ) {
						$message = $wpdb->print_error();
					}
					unset($update_array, $id_array);
					if ( $message ) {
						$messages[$datetimechanged] = $message;
						$datetimechanged++;
					}
				}
				$sendback = add_query_arg( array('datetimechanged' => $datetimechanged, 'ids' => join(',', $post_ids), 'message' => join(',',  $messages)), $sendback );
			break;
			default: return;
		}

		$sendback = remove_query_arg( array('action', 'action2', 'tags_input', 'post_author', 'comment_status', 'ping_status', '_status',  'post', 'bulk_edit', 'post_view'), $sendback );
		wp_redirect($sendback);
		exit();

	}

	/* ==================================================
	 * Bulk Action for media
	 * @since	4.00
	 */
	function custom_bulk_action_media() {

		if ( !isset( $_REQUEST['detached'] ) ) {

			// get the action
			$wp_list_table = _get_list_table('WP_Media_List_Table');  
			$action = $wp_list_table->current_action();

			$allowed_actions = array("changedatetime");
			if(!in_array($action, $allowed_actions)) return;

			check_admin_referer('bulk-media');

			if(isset($_REQUEST['media'])) {
				$post_ids = array_map('intval', $_REQUEST['media']);
			}

			if(empty($post_ids)) return;

			$sendback = remove_query_arg( array('datetimechanged', 'message', 'untrashed', 'deleted', 'ids'), wp_get_referer() );
			if ( ! $sendback )
			$sendback = admin_url( "upload.php?post_type=$post_type" );

			$pagenum = $wp_list_table->get_pagenum();
			$sendback = add_query_arg( 'paged', $pagenum, $sendback );

			switch($action) {
				case 'changedatetime':
					$datetimechanged = 0;
					$postdatetimechange_datetimes = $this->sanitize_array($_REQUEST['postdatetimechange_datetime']);

					global $wpdb;
					$messages = array();
					foreach( $post_ids as $post_id ) {
						$postdate = $postdatetimechange_datetimes[$post_id].':00';
						$postdategmt = get_gmt_from_date($postdate);
						// Change DB Attachement post
						$message = 'success';
						$update_array = array(
										'post_date' => $postdate,
										'post_date_gmt' => $postdategmt,
										'post_modified' => $postdate,
										'post_modified_gmt' => $postdategmt
									);
						$id_array= array('ID'=> $post_id);
						$wpdb->show_errors();
						$wpdb->update( $wpdb->posts, $update_array, $id_array, array('%s'), array('%d') );
						if ( $wpdb->last_error !== '' ) {
							$message = $wpdb->print_error();
						}
						unset($update_array, $id_array);
						if ( $message ) {
							$messages[$datetimechanged] = $message;
							$datetimechanged++;
						}
					}
					$sendback = add_query_arg( array('datetimechanged' => $datetimechanged, 'ids' => join(',', $post_ids), 'message' => join(',',  $messages)), $sendback );
				break;
				default: return;
			}

			$sendback = remove_query_arg( array('action', 'action2', 'tags_input', 'post_author', 'comment_status', 'ping_status', '_status',  'post', 'bulk_edit', 'post_view'), $sendback );
			wp_redirect($sendback);
			exit();

		}

	}

	/* ==================================================
	 * Bulk Action Message
	 * @since	4.00
	 */
	function custom_bulk_admin_notices() {

	    global $post_type, $pagenow;

		if ( ($pagenow == 'upload.php' || $pagenow == 'edit.php') && isset($_REQUEST['datetimechanged']) && intval($_REQUEST['datetimechanged']) > 0 && isset($_REQUEST['message']) ) {
			$messages = explode(',',sanitize_text_field(urldecode($_REQUEST['message'])));
			$success_count = 0;
			foreach ( $messages as $message ) {
				if ( $message === 'success' ) {
					++$success_count;
				} else {
					echo '<div class="notice notice-error is-dismissible"><ul><li>'.$message.'</li></ul></div>';
				}
			}
			if ( $success_count > 0 ) {
				echo '<div class="notice notice-success is-dismissible"><ul><li>'.sprintf(__('%1$d updated.', 'post-date-time-change'),$success_count).'</li></ul></div>';
			}
		}

	}

	/* ==================================================
	* Sanitize Array
	* @param	array	$a
	* @return	string	$_a
	* @since	4.07
	*/
	function sanitize_array($a) {

		$_a = array();
		foreach($a as $key=>$value) {
			if ( is_array($value) ) {
				$_a[$key] = $this->sanitize_array($value);
			} else {
				$_a[$key] = htmlspecialchars($value);
			}
		}

		return $_a;

	}

}

?>